% Init environment 
global landmarks world_size
landmarks  = [20.0 20.0; 80.0 80.0; 20.0 80.0; 80.0 20.0];
world_size = [100 100];

% Init noise values
sm = 0.1; % motion noise
st = 0.05; % rotation noise
sz = 3;    % measurement noise

%% Init robot
%myrobot = robot(rand*world_size(1), rand*world_size(2), rand*2*pi);
myrobot = robot(30, 30, pi/6);
myrobot = myrobot.set_noise(sm, st, sz);



%% Initialize particle set
N = 600; % number of particles
p = cell(N,1);
for i = 1:N
    p{i} = robot(rand*world_size(1), rand*world_size(2), rand*2*pi);
    p{i} = p{i}.set_noise(sm, st, sz);
end
w = ones(N,1) / N;

h = figure(1);
clf(h);
plot_world(myrobot, p);
meanError = eval_pf(myrobot, p)

%% Run localization experiment 
T = 10; % number of time steps
for t = 1:T
    turn = 0.1;
    motion = 5.0;
    myrobot = myrobot.move(turn, motion);
    Z = myrobot.sense();

    % Sample particle set from motion model
    fprintf('Iteration %d:\nMoving particles...', t); tic;
    for i = 1:N
        p{i} = p{i}.move(turn, motion);
    end
    fprintf('took %.4f s\n', toc);
    
    % Compute particles weights based on measurement model
    fprintf('Computing particles weights...'); tic;
    for i = 1:N
        w(i) = p{i}.measurement_prob(Z);
    end
    fprintf('took %.4f s\n', toc);
    
    % Resample
    fprintf('Resampling...'); tic;
    [p, w] = pf_resample2(p, w);
    fprintf('took %.4f s\n', toc);
    
    fprintf('Drawing...'); tic;
    clf(h);
    plot_world(myrobot, p);
    meanError = eval_pf(myrobot, p);
    fprintf('Took %.4f s, mean error= %.4f\n\n', toc, meanError);
    pause(0.5);
end


